#include "motor.hpp"
#include "../osHandles.h"
#include "../tasks/bluetooth.hpp"
#include "../drivers/uart1.h"
#include "../tasks/blinkTask.hpp"
#include "utils/rprintf.h"
#include "drivers/i2c.h"

void initialize_PWM()
{
	//select pin 0.7 for pwm 2

	PINSEL0 &= ~(3 << 14);//clear bits 14 15
	PINSEL0 |= (2 << 14);//set bits 14 15 to 10

	//select pin 0.21 for pwm 5

	PINSEL1 &= ~(3 << 10);//clear bits 10 11
	PINSEL1 |= (1 << 10);//set bits 01 11 to 10

	//set direction as output

	IODIR0 |= (1 << 7); //set pin direction for pin 0.7 as output
	IODIR0 |= (1 << 21); //set pin direction for pin 0.21 as output

	//now set pwm control

	PWMPR = 0; // Set Pre Scale to 0
	PWMPCR = 0; // PWM control register disable and deselect all bits channels single edge controlled pwm4
	PWMPCR |= (1 << 10) | (1 << 13); //set pwm2 and pwm5output enable

	//set match registers
	PWMMR0 = 0x00000340; // Max Value of Pulse Width 15mhz/340h=19kHz

	/********************************
	 * SPEED REGULATION
	 below is the value to change to increase the time the signal is high
	 convert hex value to decimal out of 256
	 time high 20h = 32 out of 256
	 *******************************/

	PWMMCR = 0x00000002; // Reset PWMMTC when Matched with PWMMR0
	PWMLER = 0x7F; // Latch in all PWMMMR's
	PWMTCR = 0x02; // Reset PWMMTC, (bit1)
	PWMTCR = 0x09; // Counter Enable, PWM Mode Enabled(bit0, bit3)}

}

void Config_Motor_Pins()
{
	//motor control for pwm2
	PINSEL0 &= ~(3 << 20);//clear bits 20 and 21
	PINSEL0 &= ~(3 << 22);//clear bits 22 and 23
	IODIR0 |= (1 << 10); //set pin direction for pin 10 as output
	IODIR0 |= (1 << 11); //set pin direction for pin 11 as output

	//motor control for pwm5
	PINSEL0 &= ~(3 << 24);//clear bits 24 and 25
	PINSEL0 &= ~(3 << 26);//clear bits 26 and 27
	IODIR0 |= (1 << 12); //set pin direction for pin 12 as output
	IODIR0 |= (1 << 13); //set pin direction for pin 13 as output
}

void setPWM()
{
	PWMMCR = 0x00000002; // Reset PWMMTC when Matched with PWMMR0
	PWMLER = 0x7F; // Latch in all PWMMMR's
	PWMTCR = 0x02; // Reset PWMMTC, (bit1)
	PWMTCR = 0x09; // Counter Enable, PWM Mode Enabled(bit0, bit3)
}

void Forward()
{
	//pwm2
	IOSET0 = (1 << 10);
	IOCLR0 = (1 << 11);

	//pwm5
	IOSET0 = (1 << 13);
	IOCLR0 = (1 << 12);
}

void backward()
{
	//pmw2
	IOSET0 = (1 << 11);
	IOCLR0 = (1 << 10);

	//pmw5
	IOSET0 = (1 << 12);
	IOCLR0 = (1 << 13);
}
void Donut_left()
{
	//pmw2
	IOCLR0 = (1 << 11);//high
	IOCLR0 = (1 << 10);//low

	//pmw5
	IOSET0 = (1 << 12);//high
	IOCLR0 = (1 << 13);//low
}

void Donut_right()
{
	//pmw2
	IOSET0 = (1 << 11);
	IOCLR0 = (1 << 10);

	//pmw5
	IOCLR0 = (1 << 12);
	IOCLR0 = (1 << 13);
}
void stop()
{
	//pmw2
	IOCLR0 = (1 << 11);
	IOCLR0 = (1 << 10);

	//pmw5
	IOCLR0 = (1 << 12);
	IOCLR0 = (1 << 13);
}

void motor(void* p)
{
	OSHANDLES *osHandles = (OSHANDLES*) p;
	initialize_PWM();
	Config_Motor_Pins();

	const char forward = 'F';
	const char forward_left = 'L';
	const char forward_right = 'R';
	const char backwards = 'B';
	const char backwards_left = 'S';
	const char backwards_right = 'T';
	const char donut_right = 'D';
	const char donut_left = 'C';
	const char fast = 'X';
	const char slow = 'W';
	int speed = 0;//start speed slow
	char SwitchData = 0;
	PWMMR2 = 0; // PWM2 Match Value //this is the value to change SPEEEEEEEED
	PWMMR5 = 0;

	for (;;)
	{
		char name;
		int timer = 0;
		int MovingTime = 500;
		if (xQueueReceive(osHandles->queue.command, &name, portMAX_DELAY))
		{
			SwitchData = name;
			rprintf("switch data is motor task is %c", SwitchData);
			switch (SwitchData)
			{
			case forward:
				rprintf("forward case speed is: %i\n\n", speed);
				if (speed==1)
				{
					PWMMR2 = 0x00000280; // PWM2 Match Value //this is the value to change SPEEEEEEEED
					PWMMR5 = 0x00000295;
				}
				else
				{
					PWMMR2 = 0x00000080; // PWM2 Match Value //this is the value to change SPEEEEEEEED
					PWMMR5 = 0x00000095;
				}
				setPWM();
				Forward();

				while (timer < MovingTime)
				{
					vTaskDelay(1);
					timer++;
				}
				timer = 0;
				stop();

				break;
			case forward_left:
				if (speed == 1)
				{
					PWMMR2 = 0x00000210; // PWM2 Match Value //this is the value to change SPEEEEEEEED
					PWMMR5 = 0x00000140; // PWM5 Match Value //this is the value to change SPEEEEEEEED
				}
				else
				{
					PWMMR2 = 0x00000115; // PWM2 Match Value //this is the value to change SPEEEEEEEED
					PWMMR5 = 0x00000070; // PWM5 Match Value //this is the value to change SPEEEEEEEED
				}

				setPWM();
				Forward();

				while (timer < MovingTime)
				{
					vTaskDelay(1);
					timer++;
				}
				timer = 0;
				stop();

				break;

			case forward_right:
				rprintf("in case forward right");
				if (speed == 1)
				{
					PWMMR2 = 0x00000140; // PWM2 Match Value SPEED
					PWMMR5 = 0x00000210; // PWM5 Match Value SPEED
				}
				else
				{
					PWMMR2 = 0x00000070; // PWM2 Match Value SPEED
					PWMMR5 = 0x00000115; // PWM5 Match Value SPEED
				}

				setPWM();
				Forward();
				while (timer < MovingTime)
				{
					vTaskDelay(1);
					timer++;
				}
				timer = 0;
				stop();
				break;

			case backwards:
				if (speed == 1)
				{
					PWMMR2 = 0x00000175; // PWM2 Match Value SPEED
					PWMMR5 = 0x00000200; // PWM5 Match Value SPEED
				}
				else
				{
					PWMMR2 = 0x00000076; // PWM2 Match Value SPEED
					PWMMR5 = 0x00000098; // PWM5 Match Value SPEED
				}
				setPWM();
				backward();

				while (timer < MovingTime)
				{
					vTaskDelay(1);
					timer++;
				}
				timer = 0;
				stop();
				break;

			case backwards_left:
				if (speed == 1)
				{
					PWMMR2 = 0x00000220; // PWM2 Match Value SPEED
					PWMMR5 = 0x00000100; // PWM5 Match Value SPEED
				}
				else
				{
					PWMMR2 = 0x00000175; // PWM2 Match Value SPEED
					PWMMR5 = 0x00000050; // PWM5 Match Value SPEED
				}
				setPWM();
				backward();
				while (timer < MovingTime)
				{
					vTaskDelay(1);
					timer++;
				}
				timer = 0;
				stop();
				break;
			case backwards_right:
				if (speed == 1)
				{
					PWMMR2 = 0x00000100; // PWM2 Match Value SPEED
					PWMMR5 = 0x00000220; // PWM5 Match Value SPEED
				}
				else
				{
					PWMMR2 = 0x00000050; // PWM2 Match Value SPEED
					PWMMR5 = 0x00000175; // PWM5 Match Value SPEED
				}
				setPWM();
				backward();
				while (timer < MovingTime)
				{
					vTaskDelay(1);
					timer++;
				}
				timer = 0;
				stop();
				break;

			case donut_right:
			if (speed == 1)
			{
				PWMMR2 = 0x00000200; // PWM2 Match Value SPEED
				PWMMR5 = 0x00000100; // PWM5 Match Value SPEED
			}
			else
			{
				PWMMR2 = 0x000000150; // PWM2 Match Value SPEED
				PWMMR5 = 0x00000075; // PWM5 Match Value SPEED
			}

			setPWM();
			Donut_right();
			while (timer < 2500)
			{
				vTaskDelay(1);
				timer++;
			}
				timer = 0;
				stop();
				break;

			case donut_left:
			if (speed == 1)
			{
				PWMMR2 = 0x00000100; // PWM2 Match Value SPEED
				PWMMR5 = 0x00000220; // PWM5 Match Value SPEED
			}
			else
			{
				PWMMR2 = 0x00000050; // PWM2 Match Value SPEED
				PWMMR5 = 0x00000175; // PWM5 Match Value SPEED
			}

			setPWM();
			Donut_left();
			while (timer < 2500)
			{
				vTaskDelay(1);
				timer++;
			}
			timer = 0;
			stop();
			break;

			case fast:
				rprintf("in fast speed case\n\n");
				speed = 1;
				break;
			case slow:
				speed = 0;
				break;

			default:
				break;

			}
		}
		else
			rprintf("Nothing Received");
	}
}
